#!BPY
# vim:encoding=ISO-8859-1

"""
RGBA-L{Color} class (with HSVA conversion support etc.) and some default colors.

@version: 0.2
@author:  Mathias Panzenbck
@license: LGPL

@group Helper Functions: getCurrentColor, hextoint

@group Default Colors: RED, GREEN, BLUE, WHITE, BLACK,
	GREY, LIGHTGREY, DARKGREY, YELLOW, ORANGE

@var HTML_COLORS: A dictionary of all default HTML-colors.
@var RED:         default color: red
@var GREEN:       default color: green
@var BLUE:        default color: blue
@var YELLOW:      default color: yellow
@var ORANGE:      default color: orange
@var GREY:        default color: grey
@var LIGHTGREY:   default color: lightgrey
@var DARKGREY:    default color: darkgrey
@var WHITE:       default color: white
@var BLACK:       default color: black
"""

__author__  = "Mathias Panzenbck"
__version__ = "0.2"
__url__     = ("blender","elysiun", "http://www.elysiun.com/forum/viewtopic.php?t=38458&postdays=0&postorder=asc&start=0",)
__license__ = "LGPL"
__bpydoc__  = """\
Description: color-class, some default-colors and helper function(s).
Usage: only used by other scripts
"""

# --------------------------------------------------------------------------
# ScriptUI.Color, Copyright (C) 2005  Mathias Panzenbck
# --------------------------------------------------------------------------
# ***** BEGIN LGPL LICENSE BLOCK *****
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# ***** END LGPL LICENCE BLOCK *****
# -------------------------------------------------------------------------- 

import math
import Blender.BGL
from Blender.BGL import *

def hextoint(s):
	"""
	Convert a hex-string (without the leadig '0x') to a number.

	@type  s: string
	@param s: a hex-string

	@rtype:  int or long
	@return: number value of the hex-string.

	@raise ValueError: When hex-string is missformatted.
	"""
	d = ord('0')
	a = ord('a')
	f = ord('f')
	n = 0
	
	for c in s:
		n *= 16
		x  = c.lower()
		o  = ord(x)
		
		if x.isdigit():
			n += o - d
		elif a <= o and f >= o:
			n += 10 + o - a
		else:
			raise ValueError
	return n

def getCurrentColor():
	"""
	Returns the current color.

	@rtype:  Color
	@return: The current color.
	"""
	color = Buffer(GL_FLOAT, 4)
	glGetFloatv(GL_CURRENT_COLOR, color)
	return Color( *color.list )

class ColorIterator:
	"""
	Iterate over a L{Color} like it would be sequence.

	(Only for internal usage.)
	"""
	def __init__(self,color):
		self.color = color
		self.pos   = -1
	
	def __iter__(self):
		return self
	
	def next(self):
		self.pos += 1
		try:
			return self.color[ self.pos ]
		except IndexError:
			raise StopIteration

class Color:
	"""
	RGBA-color-class.

	This color-class stores it's color-value as red, green, blue and alpha float values.
	It supports RGB <-> HSV conversion and overloads the +, -, *, /, +=, -=, *=, /=
	operators. It also supports sequence-emulation.

	@ivar red:   red-value
	@ivar green: green-value
	@ivar blue:  blue-value
	@ivar alpha: alpha-value
	"""
	def __init__(self,red=None,green=None,blue=None,alpha=None):
		"""
		Initialise a color.

		Values are ether of float [0.,1.], int [0,255] or None (same as 1.) type.
		When all values except red are None and the type of red is long, it will be
		interpreted as packed RGBA value: 0xRRGGBBAA, where R is the red-proportion,
		G the green-proportion, B the blue-proportion and A the alpha-proportion.

		When all values except red are None and the type of red is string,
		L{Color.fromHtmlString}(red) will be called.
		"""
		self.red   = 1.
		self.green = 1.
		self.blue  = 1.
		self.alpha = 1.
		
		if green == None and blue == None and alpha == None:
			if type(red) == long:
				self.red   = ((0xff000000L & red) >> 24) / 255.
				self.green = ((0x00ff0000L & red) >> 16) / 255.
				self.blue  = ((0x0000ff00L & red) >>  8) / 255.
				self.alpha =  (0x000000ffL & red)        / 255.
			elif type(red) == str:
				self.fromHtmlString(red)
			else:
				self.set(red,green,blue,alpha)
		else:
			self.set(red,green,blue,alpha)
	
	def set(self,red,green,blue,alpha=1.):
		"""
		Set the red, green, blue, alpha values.

		Values are ether of float [0.,1.], int [0,255] or None (same as 1.) type.

		@type  red:   int, float or None
		@param red:   The new red value.
		@type  green: int, float or None
		@param green: The new green value.
		@type  blue:  int, float or None
		@param blue:  The new blue value.
		@type  alpha: int, float or None
		@param alpha: The new alpha value.
		"""
		decide = {int: lambda x: x/255., float: lambda x:x, type(None):lambda x:1.}

		self.red   = decide[type(red)  ](red)
		self.green = decide[type(green)](green)
		self.blue  = decide[type(blue) ](blue)
		self.alpha = decide[type(alpha)](alpha)
	
	def __iter__(self):
		return ColorIterator(self)

	def __len__(self):
		return 4
	
	def __getitem__(self, key):
		if   key == 0:
			return self.red
		elif key == 1:
			return self.green
		elif key == 2:
			return self.blue
		elif key == 3:
			return self.alpha
		else:
			raise IndexError, "list index out of range"
	
	def __setitem__(self, key, value):
		if   key == 0:
			self.red   = value
		elif key == 1:
			self.green = value
		elif key == 2:
			self.blue  = value
		elif key == 3:
			self.alpha = value
		else:
			raise IndexError, "list index out of range"
	
	def __repr__(self):
		return "Color(%f, %f, %f, %f)" % (self.red, self.green, self.blue, self.alpha)

	def __str__(self):
		return repr(self)

	def __hex__(self):
		return "0x%02x%02x%02x%02x" % (int(self.red * 255), int(self.green * 255), int(self.blue * 255), int(self.alpha * 255))

	def __long__(self):
		return long(
			(int(self.red   * 255) << 24) +
			(int(self.green * 255) << 16) +
			(int(self.blue  * 255) <<  8) +
			int(self.alpha * 255))

	def getIntList(self):
		"""
		Returns a list of integers.

		@rtype:  [int,int,int,int]
		@return: A list of 4 integers (red, green, blue, alpha), each in range 0 to 255.
		"""
		return [
			int(self.red   * 255),
			int(self.green * 255),
			int(self.blue  * 255),
			int(self.alpha * 255),
			]

	def getHtmlString(self):
		"""
		Returns the HTML-representation of the color.

		@rtype:  str
		@return: HTML-color-string
		"""
		return "#%02x%02x%02x" % (int(self.red * 255), int(self.green * 255), int(self.blue * 255))
	
	def getHtmlStringWithAlpha(self):
		"""
		Returns a HTML  like representation of the color with alpha value.
		
		@rtype:  str
		@return: HTML like color-string
		"""
		return "#%02x%02x%02x%02x" % (int(self.red * 255), int(self.green * 255), int(self.blue * 255), int(self.alpha * 255))

	def fromHtmlString(self,s):
		"""
		Converts a HTML-color-string to a Color.

		When C{len(s) == 9}, the last two hex-digits will be used as alpha value.
		When the string is missformated, ValueError will be raised.

		@type  s: string
		@param s: The HTML-color-string.

		@raise ValueError: When the HTML-string is missformated.
		"""
		l = len(s)
		
		if l == 7 or l == 9:
			if s[0] == '#':
				self.red   = hextoint(s[1:3]) / 255.
				self.green = hextoint(s[3:5]) / 255.
				self.blue  = hextoint(s[5:7]) / 255.

				if l == 9:
					self.alpha = hextoint(s[7:9]) / 255.
			else:
				try:
					col.fromHtmlString(HTML_COLORS[s])
				except KeyError:
					raise ValueError
		else:
			raise ValueError
			

	def __mul__(self,other):
		color = Color(*self)
		color *= other
		return color
		
	def __rmul__(self,other):
		color = Color(*self)
		color *= other
		return color
	
	def __add__(self,other):
		color = Color(*self)
		color += other
		return color
		
	def __radd__(self,other):
		color = Color(*self)
		color += other
		return color
	
	def __div__(self,other):
		color = Color(*self)
		color /= other
		return color
		
	def __rdiv__(self,other):
		color = Color(*self)
		
		color /= 1.
		color *= other
		
		return color
		
	def __sub__(self,other):
		color = Color(*self)
		color -= other
		return color
		
	def __rsub__(self,other):
		color = Color(*self)
		
		color *= -1.
		color += other
		
		return color
	
	def __imul__(self,other):
		if isinstance(other,Color):
			self.red   *= other.red
			self.green *= other.green
			self.blue  *= other.blue
			self.alpha *= other.alpha
		elif "__getitem__" in dir(other):
			self.red   *= other[0]
			self.green *= other[1]
			self.blue  *= other[2]
			self.alpha *= other[3]
		else:
			self.red   *= other
			self.green *= other
			self.blue  *= other
			self.alpha *= other
			
	def __idiv__(self,other):
		if isinstance(other,Color):
			self.red   /= other.red
			self.green /= other.green
			self.blue  /= other.blue
			self.alpha /= other.alpha
		elif "__getitem__" in dir(other):
			self.red   /= other[0]
			self.green /= other[1]
			self.blue  /= other[2]
			self.alpha /= other[3]
		else:
			self.red   /= other
			self.green /= other
			self.blue  /= other
			self.alpha /= other
			
	def __iadd__(self,other):
		if isinstance(other,Color):
			self.red   += other.red
			self.green += other.green
			self.blue  += other.blue
			self.alpha += other.alpha
		elif "__getitem__" in dir(other):
			self.red   += other[0]
			self.green += other[1]
			self.blue  += other[2]
			self.alpha += other[3]
		else:
			self.red   += other
			self.green += other
			self.blue  += other
			self.alpha += other
			
	def __isub__(self,other):
		if isinstance(other,Color):
			self.red   -= other.red
			self.green -= other.green
			self.blue  -= other.blue
			self.alpha -= other.alpha
		elif "__getitem__" in dir(other):
			self.red   -= other[0]
			self.green -= other[1]
			self.blue  -= other[2]
			self.alpha -= other[3]
		else:
			self.red   -= other
			self.green -= other
			self.blue  -= other
			self.alpha -= other

	def __eq__(self,other):
		for i in xrange(len(self)):
			if self[i] != other[i]:
				return False
		return True
	
	def __ne__(self,other):
		return not (self == other)

	def toHSV(self):
		"""
		Convert RGBA color to HSVA.

		@rtype:  [float,float,flaot,float]
		@return: HSVA-color.
		"""
		minVal = min(self.red, self.green, self.blue)
		V      = max(self.red, self.green, self.blue)

		Delta = V - minVal

		# Calculate saturation: saturation is 0 if r, g and b are all 0
		if V == 0.0:
			S = 0.0
		else:
			S = Delta / V

		if S == 0.0:
			H = 0.0           # Achromatic: When s = 0, h is undefined but who cares
		# Chromatic:
		elif self.red == V:   # between yellow and magenta [degrees]
			H = (1./6.) * (self.green - self.blue) / Delta
		elif self.green == V: # between cyan and yellow
			H = (1./3.) + (1./6.) * (self.blue - self.red) / Delta
		else:                 # between magenta and cyan
			H = (2./3.) + (1./6.) * (self.red - self.green) / Delta
			
		if H < 0.0:
			H += 1.
		# return a list of values as an rgb object would not be sensible
		return [H, S, V, self.alpha]

	def fromHSV(self,h,s,v,alpha=1.):
		"""
		Converts HSVA color to RGBA.

		@type  h: float
		@param h: hue
		@type  s: float
		@param s: saturtation
		@type  v: float
		@param v: value
		@type  alpha: float
		@param alpha: alpha-value
		"""
		self.alpha = float(alpha)

		h = float(h)
		s = float(s)
		v = float(v)
		hTemp = 0.0

		if s == 0.0:               # color is on black-and-white center line
			self.red   = v         # achromatic: shades of gray
			self.green = v         # supposedly invalid for h=0 but who cares
			self.blue  = v
		# chromatic color:
		else:
			if h == 1.0:           # 360 degrees same as 0 degrees
				hTemp = 0.0
			else:
				hTemp = h

		hTemp = hTemp * 6.        # h is now in [0,6)
		i     = math.floor(hTemp) # largest integer <= h
		f     = hTemp - i         # fractional part of h

		p = v * (1.0 - s)
		q = v * (1.0 - (s * f))
		t = v * (1.0 - (s * (1.0 - f)))

		if i == 0:
			self.red   = v
			self.green = t
			self.blue  = p
		elif i == 1:
			self.red   = q
			self.green = v
			self.blue  = p
		elif i == 2:
			self.red   = p
			self.green = v
			self.blue  = t
		elif i == 3:
			self.red   = p
			self.green = q
			self.blue  = v
		elif i == 4:
			self.red   = t
			self.green = p
			self.blue  = v
		elif i == 5:
			self.red   = v
			self.green = p
			self.blue  = q

HTML_COLORS = {
	'black':   '#000000',
	'gray':    '#808080',
	'maroon':  '#800000',
	'red':     '#FF0000',
	'green':   '#008000',
	'lime':    '#00FF00',
	'olive':   '#808000',
	'yellow':  '#FFFF00',
	'navy':    '#000080',
	'blue':    '#0000FF',
	'purple':  '#800080',
	'fuchsia': '#FF00FF',
	'teal':    '#008080',
	'aqua':    '#00FFFF',
	'silver':  '#C0C0C0',
	'white':   '#FFFFFF'
	}

RED       = Color(1.,0.,0.,1.)
GREEN     = Color(0.,1.,0.,1.)
BLUE      = Color(0.,0.,1.,1.)
YELLOW    = Color(1.,1.,0.,1.)
ORANGE    = Color(1.,.5,0.,1.)
GREY      = Color(.5,.5,.5,1.)
LIGHTGREY = Color(.75,.75,.75,1.)
DARKGREY  = Color(.25,.25,.25,1.)
WHITE     = Color(1.,1.,1.,1.)
BLACK     = Color(.0,.0,.0,1.)
