#!BPY
# vim:encoding=ISO-8859-1

"""
Popups for ScriptUI.

@version: 0.9.1
@author:  Mathias Panzenb�ck
@license: LGPL
"""

__author__  = "Mathias Panzenb�ck"
__version__ = "0.9.1"
__url__     = ("blender","elysiun","http://www.elysiun.com/forum/viewtopic.php?t=38458&postdays=0&postorder=asc&start=0",)
__license__ = "LGPL"
__bpydoc__  = """\
Description: Wrapper to blenders script-UI-eventsystem
Usage: only used by other scripts
"""

# --------------------------------------------------------------------------
# ScriptUI.Popups, Copyright (C) 2005  Mathias Panzenb�ck
# --------------------------------------------------------------------------
# ***** BEGIN LGPL LICENSE BLOCK *****
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# ***** END LGPL LICENCE BLOCK *****
# -------------------------------------------------------------------------- 

import Blender
from   Blender.BGL    import *
from   Blender.Draw   import *
from   Blender.Window import *

import ScriptUI
from ScriptUI import *

class Popup:
	"""
	Base-class for all kinds of popups.
	"""
	def __init__(self):
		pass
	
	def show(self):
		"""
		Show the popup.
		"""
		return None

	def __call__(self):
		"""
		Same as L{Popup.show}.
		"""
		return self.show()

class FloatInput(Popup):
	"""
	Create a float-input popup.
	"""
	def __init__(self, text, default, min, max, clickStep, floatLen):
		"""
		@param text: The text that is displayed in the popup.
		@type  text: string
		@param default: The value that the popup is set to initially.
		@type  default: float
		@param min: The lowest value the popup will allow.
		@type  min: float
		@param max: The highest value the popup will allow.
		@type  max: float
		@param clickStep: How much is incremented per user click, 100 will increment 1.0, 10 will increment 0.1 etc.
		@type  clickStep: int 
		@param floatLen: The number of decimal places to display, between 2 and 4.
		@type  floatLen: int
		"""
		self.text      = str(text)
		self.default   = float(default)
		self.min       = float(min)
		self.max       = float(max)
		self.clickStep = float(clickStep)
		self.floatLen  = float(floatLen)
	
	def show(self):
		"""
		Show the popup.

		@rtype: float or None
		@return: The input value or None on cancel.
		"""
		return Blender.Draw.PupFloatInput(
				self.text, self.default, self.min, self.max,
				self.clickStep, self.floatLen)

class IntInput(Popup):
	"""
	Create a int-input popup.
	"""
	def __init__(self, text, default, min, max):
		"""
		@param text: The text that is displayed in the popup.
		@type  text: string 
		@param default: The value that the popup is set to initially.
		@type  default: int 
		@param min: The lowest value the popup will allow.
		@type  min: int 
		@param max: The highest value the popup will allow.
		@type  max: int 
		"""
		self.text    = str(text)
		self.default = int(default)
		self.min     = int(min)
		self.max     = int(max)
	
	def show(self):
		"""
		Show the popup.

		@rtype: int or None
		@return: The input value or None on cancel.
		"""
		return Blender.Draw.PupIntInput(
				self.text, self.default, self.min, self.max)

class StrInput(Popup):
	"""
	Create a string-input popup.
	"""
	def __init__(self, text, default, max=20):
		"""
		@param text: The text that is displayed in the popup.
		@type  text: string 
		@param default: The value that the popup is set to initially.
			If it's longer then 'max', it's truncated.
		@type  default: string 
		@param max: The most characters the popup input will allow.
			If not given it defaults to 20 chars. It should be in the range [1, 100].
		@type  max: int 
		"""
		self.text    = str(text)
		self.default = str(default)
		self.max     = int(max)
	
	def show(self):
		"""
		Show the popup.

		@rtype: string or None
		@return: The input value or None on cancel.
		"""
		return Blender.Draw.PupStrInput(
				self.text, self.default, self.max)

class Menu(Popup):
	"""
	Create a menu-popup.
	
	@cvar ITEM: Defines a menu-entry as "item".
	@type ITEM: int
	@cvar SEP:  Defines a menu-entry as "seperator".
	@type SEP:  int
	"""
	
	ITEM = MenuBase.ITEM
	SEP  = MenuBase.SEP
	
	def __init__(self, name=None, maxrow=None, items=[], cancel_clbk=NOOP):
		"""
		@type  name: string
		@param name: The name (title) of the menu.
		@type  maxrow: int
		@param maxrow: The maximum number of rows for each column in the pop-up. 
		@type  items: list
		@param items: Menu-items (see: L{ScriptUI.MenuBase})
		@type  cancel_clbk: function with no parameter
		@param cancel_clbk: This function will be called if the selection will be canceld.
		"""
		self.menu   = MenuBase(name,items,cancel_clbk)
		self.maxrow = maxrow
	
	def addItem(self,name,clbk=NOOP):
		"""
		Add a new menu-entry.

		@type  name: string
		@param name: The name (title) of the new menu-entry.
		@type  clbk: function with no parameters
		@param clbk: The function to call when the menu-entry was clicked.
		"""
		self.menu.addItem(name,clbk)

	def addSep(self):
		"""
		Add a new seperator.
		"""
		self.menu.addSep()
	
	def show(self):
		"""
		Show the popup.

		@rtype:  int
		@return: Index of choosen menu-entry or -1 on cancel.
		"""
		if self.maxrow == None:
			ind = Blender.Draw.PupMenu(self.menu.get_menu_str())
		else:
			ind = Blender.Draw.PupMenu(self.menu.get_menu_str(), self.maxrow)

		self.menu( ind )

		return ind

class MsgBox(Popup):
	"""
	Create a message-box (one-line menu without title).

	Use C{Blender.Draw.PupMenu("message")} when there is no need to store the message.
	"""
	def __init__(self,message):
		"""
		@type  message: string
		@param message: The popup-message.
		"""
		self.message = str(message)
	
	def show(self):
		"""
		Show the popup.
		"""
		Blender.Draw.PupMenu(self.message)
